@@ -30,12 +30,11 @@ import Distribution.Client.Targets
          ( userToPackageConstraint )
 
 import Distribution.Simple.Compiler
-         ( CompilerId(..), Compiler(compilerId)
-         , PackageDB(..), PackageDBStack )
+         ( Compiler, CompilerInfo, compilerInfo, PackageDB(..), PackageDBStack )
 import Distribution.Simple.Program (ProgramConfiguration )
 import Distribution.Simple.Setup
          ( ConfigFlags(..), fromFlag, toFlag, flagToMaybe, fromFlagOrDefault )
-import Distribution.Simple.PackageIndex (PackageIndex)
+import Distribution.Simple.PackageIndex (InstalledPackageIndex)
 import Distribution.Simple.Utils
          ( defaultPackageDesc )
 import qualified Distribution.InstalledPackageInfo as Installed
@@ -64,6 +63,8 @@ chooseCabalVersion :: ConfigExFlags -> Maybe Version -> VersionRange
 chooseCabalVersion configExFlags maybeVersion =
   maybe defaultVersionRange thisVersion maybeVersion
   where
+    -- Cabal < 1.19.2 doesn't support '--exact-configuration' which is needed
+    -- for '--allow-newer' to work.
     allowNewer = fromFlagOrDefault False $
                  fmap isAllowNewer (configAllowNewer configExFlags)
 
@@ -125,6 +126,7 @@ configure verbosity packageDBs repos comp platform conf
                            (configDistPref configFlags),
       useLoggingHandle = Nothing,
       useWorkingDir    = Nothing,
+      useWin32CleanHack        = False,
       forceExternalSetupMethod = False,
       setupCacheLock   = Nothing
     }
@@ -147,13 +149,13 @@ configure verbosity packageDBs repos comp platform conf
 planLocalPackage :: Verbosity -> Compiler
                  -> Platform
                  -> ConfigFlags -> ConfigExFlags
-                 -> PackageIndex
+                 -> InstalledPackageIndex
                  -> SourcePackageDb
                  -> IO (Progress String String InstallPlan)
 planLocalPackage verbosity comp platform configFlags configExFlags installedPkgIndex
   (SourcePackageDb _ packagePrefs) = do
   pkg <- readPackageDescription verbosity =<< defaultPackageDesc verbosity
-  solver <- chooseSolver verbosity (fromFlag $ configSolver configExFlags) (compilerId comp)
+  solver <- chooseSolver verbosity (fromFlag $ configSolver configExFlags) (compilerInfo comp)
 
   let -- We create a local package and ask to resolve a dependency on it
       localPkg = SourcePackage {
@@ -200,7 +202,7 @@ planLocalPackage verbosity comp platform configFlags configExFlags installedPkgI
             (SourcePackageDb mempty packagePrefs)
             [SpecificSourcePackage localPkg]
 
-  return (resolveDependencies platform (compilerId comp) solver resolverParams)
+  return (resolveDependencies platform (compilerInfo comp) solver resolverParams)
 
 
 -- | Call an installer for an 'SourcePackage' but override the configure
@@ -212,7 +214,7 @@ planLocalPackage verbosity comp platform configFlags configExFlags installedPkgI
 -- NB: when updating this function, don't forget to also update
 -- 'installReadyPackage' in D.C.Install.
 configurePackage :: Verbosity
-                 -> Platform -> CompilerId
+                 -> Platform -> CompilerInfo
                  -> SetupScriptOptions
                  -> ConfigFlags
                  -> ReadyPackage
