@@ -11,6 +11,7 @@
 -- Extra utils related to the package indexes.
 -----------------------------------------------------------------------------
 module Distribution.Client.IndexUtils (
+  getIndexFileAge,
   getInstalledPackages,
   getSourcePackages,
   getSourcePackagesStrict,
@@ -20,6 +21,7 @@ module Distribution.Client.IndexUtils (
   parsePackageIndex,
   readRepoIndex,
   updateRepoIndexCache,
+  updatePackageIndexCacheFile,
 
   BuildTreeRefType(..), refTypeFromTypeCode, typeCodeFromRefType
   ) where
@@ -33,6 +35,7 @@ import Distribution.Package
          , Dependency(Dependency), InstalledPackageId(..) )
 import Distribution.Client.PackageIndex (PackageIndex)
 import qualified Distribution.Client.PackageIndex      as PackageIndex
+import Distribution.Simple.PackageIndex (InstalledPackageIndex)
 import qualified Distribution.Simple.PackageIndex      as InstalledPackageIndex
 import qualified Distribution.InstalledPackageInfo     as InstalledPackageInfo
 import qualified Distribution.PackageDescription.Parse as PackageDesc.Parse
@@ -55,7 +58,7 @@ import Distribution.Text
 import Distribution.Verbosity
          ( Verbosity, normal, lessVerbose )
 import Distribution.Simple.Utils
-         ( die, warn, info, fromUTF8, tryFindPackageDesc )
+         ( die, warn, info, fromUTF8 )
 
 import Data.Char   (isAlphaNum)
 import Data.Maybe  (mapMaybe, fromMaybe)
@@ -69,7 +72,8 @@ import qualified Data.ByteString.Lazy.Char8 as BS.Char8
 import qualified Data.ByteString.Char8 as BSS
 import Data.ByteString.Lazy (ByteString)
 import Distribution.Client.GZipUtils (maybeDecompress)
-import Distribution.Client.Utils (byteStringToFilePath)
+import Distribution.Client.Utils ( byteStringToFilePath
+                                 , tryFindAddSourcePackageDesc )
 import Distribution.Compat.Exception (catchIO)
 import Distribution.Client.Compat.Time (getFileAge, getModTime)
 import System.Directory (doesFileExist)
@@ -79,18 +83,19 @@ import System.FilePath.Posix as FilePath.Posix
 import System.IO
 import System.IO.Unsafe (unsafeInterleaveIO)
 import System.IO.Error (isDoesNotExistError)
+import Numeric (showFFloat)
 
 
 getInstalledPackages :: Verbosity -> Compiler
                      -> PackageDBStack -> ProgramConfiguration
-                     -> IO InstalledPackageIndex.PackageIndex
+                     -> IO InstalledPackageIndex
 getInstalledPackages verbosity comp packageDbs conf =
     Configure.getInstalledPackages verbosity' comp packageDbs conf
   where
     --FIXME: make getInstalledPackages use sensible verbosity in the first place
     verbosity'  = lessVerbose verbosity
 
-convert :: InstalledPackageIndex.PackageIndex -> PackageIndex InstalledPackage
+convert :: InstalledPackageIndex -> PackageIndex InstalledPackage
 convert index' = PackageIndex.fromList
     -- There can be multiple installed instances of each package version,
     -- like when the same package is installed in the global & user DBs.
@@ -178,10 +183,9 @@ readRepoIndex verbosity repo mode =
   let indexFile = repoLocalDir repo </> "00-index.tar"
       cacheFile = repoLocalDir repo </> "00-index.cache"
   in handleNotFound $ do
-    warnIfIndexIsOld indexFile
+    warnIfIndexIsOld =<< getIndexFileAge repo
     whenCacheOutOfDate indexFile cacheFile $ do
-      info verbosity "Updating the index cache file..."
-      updatePackageIndexCacheFile indexFile cacheFile
+      updatePackageIndexCacheFile verbosity indexFile cacheFile
     readPackageIndexCacheFile mkAvailablePackage indexFile cacheFile mode
 
   where
@@ -212,23 +216,27 @@ readRepoIndex verbosity repo mode =
       else ioError e
 
     isOldThreshold = 15 --days
-    warnIfIndexIsOld indexFile = do
-      dt <- getFileAge indexFile
+    warnIfIndexIsOld dt = do
       when (dt >= isOldThreshold) $ case repoKind repo of
         Left  remoteRepo -> warn verbosity $
              "The package list for '" ++ remoteRepoName remoteRepo
-          ++ "' is " ++ show dt ++ " days old.\nRun "
+          ++ "' is " ++ showFFloat (Just 1) dt " days old.\nRun "
           ++ "'cabal update' to get the latest list of available packages."
         Right _localRepo -> return ()
 
+
+-- | Return the age of the index file in days (as a Double).
+getIndexFileAge :: Repo -> IO Double
+getIndexFileAge repo = getFileAge $ repoLocalDir repo </> "00-index.tar"
+
+
 -- | It is not necessary to call this, as the cache will be updated when the
 -- index is read normally. However you can do the work earlier if you like.
 --
 updateRepoIndexCache :: Verbosity -> Repo -> IO ()
 updateRepoIndexCache verbosity repo =
     whenCacheOutOfDate indexFile cacheFile $ do
-      info verbosity "Updating the index cache file..."
-      updatePackageIndexCacheFile indexFile cacheFile
+      updatePackageIndexCacheFile verbosity indexFile cacheFile
   where
     indexFile = repoLocalDir repo </> "00-index.tar"
     cacheFile = repoLocalDir repo </> "00-index.cache"
@@ -241,7 +249,7 @@ whenCacheOutOfDate origFile cacheFile action = do
     else do
       origTime  <- getModTime origFile
       cacheTime <- getModTime cacheFile
-      when (origTime >= cacheTime) action
+      when (origTime > cacheTime) action
 
 ------------------------------------------------------------------------
 -- Reading the index file
@@ -351,7 +359,8 @@ extractPkg entry blockNo = case Tar.entryContent entry of
     | Tar.isBuildTreeRefTypeCode typeCode ->
       Just $ do
         let path   = byteStringToFilePath content
-        cabalFile <- tryFindPackageDesc path
+            err = "Error reading package index."
+        cabalFile <- tryFindAddSourcePackageDesc path err
         descr     <- PackageDesc.Parse.readPackageDescription normal cabalFile
         return $ BuildTreeRef (refTypeFromTypeCode typeCode) (packageId descr)
                               descr path blockNo
@@ -378,8 +387,9 @@ parsePreferredVersions = mapMaybe simpleParse
 -- Reading and updating the index cache
 --
 
-updatePackageIndexCacheFile :: FilePath -> FilePath -> IO ()
-updatePackageIndexCacheFile indexFile cacheFile = do
+updatePackageIndexCacheFile :: Verbosity -> FilePath -> FilePath -> IO ()
+updatePackageIndexCacheFile verbosity indexFile cacheFile = do
+    info verbosity "Updating the index cache file..."
     (mkPkgs, prefs) <- either fail return
                        . parsePackageIndex
                        . maybeDecompress
@@ -452,8 +462,9 @@ packageIndexFromCache mkPkg hnd entrs mode = accum mempty [] entrs
       -- package id for build tree references - the user might edit the .cabal
       -- file after the reference was added to the index.
       path <- liftM byteStringToFilePath . getEntryContent $ blockno
-      pkg  <- do cabalFile <- tryFindPackageDesc path
-                 PackageDesc.Parse.readPackageDescription normal cabalFile
+      pkg  <- do let err = "Error reading package index from cache."
+                 file <- tryFindAddSourcePackageDesc path err
+                 PackageDesc.Parse.readPackageDescription normal file
       let srcpkg = mkPkg (BuildTreeRef refType (packageId pkg) pkg path blockno)
       accum (srcpkg:srcpkgs) prefs entries
 
@@ -503,31 +514,32 @@ data IndexCacheEntry = CachePackageId PackageId BlockNo
                      | CachePreference Dependency
   deriving (Eq)
 
+packageKey, blocknoKey, buildTreeRefKey, preferredVersionKey :: String
+packageKey = "pkg:"
+blocknoKey = "b#"
+buildTreeRefKey     = "build-tree-ref:"
+preferredVersionKey = "pref-ver:"
+
 readIndexCacheEntry :: BSS.ByteString -> Maybe IndexCacheEntry
 readIndexCacheEntry = \line ->
   case BSS.words line of
     [key, pkgnamestr, pkgverstr, sep, blocknostr]
-      | key == packageKey && sep == blocknoKey ->
+      | key == BSS.pack packageKey && sep == BSS.pack blocknoKey ->
       case (parseName pkgnamestr, parseVer pkgverstr [],
             parseBlockNo blocknostr) of
         (Just pkgname, Just pkgver, Just blockno)
           -> Just (CachePackageId (PackageIdentifier pkgname pkgver) blockno)
         _ -> Nothing
-    [key, typecodestr, blocknostr] | key == buildTreeRefKey ->
+    [key, typecodestr, blocknostr] | key == BSS.pack buildTreeRefKey ->
       case (parseRefType typecodestr, parseBlockNo blocknostr) of
         (Just refType, Just blockno)
           -> Just (CacheBuildTreeRef refType blockno)
         _ -> Nothing
 
-    (key: remainder) | key == preferredVersionKey ->
+    (key: remainder) | key == BSS.pack preferredVersionKey ->
       fmap CachePreference (simpleParse (BSS.unpack (BSS.unwords remainder)))
     _  -> Nothing
   where
-    packageKey = BSS.pack "pkg:"
-    blocknoKey = BSS.pack "b#"
-    buildTreeRefKey     = BSS.pack "build-tree-ref:"
-    preferredVersionKey = BSS.pack "pref-ver:"
-
     parseName str
       | BSS.all (\c -> isAlphaNum c || c == '-') str
                   = Just (PackageName (BSS.unpack str))
@@ -554,13 +566,20 @@ readIndexCacheEntry = \line ->
         _   -> Nothing
 
 showIndexCacheEntry :: IndexCacheEntry -> String
-showIndexCacheEntry entry = case entry of
-   CachePackageId pkgid b -> "pkg: " ++ display (packageName pkgid)
-                                  ++ " " ++ display (packageVersion pkgid)
-                          ++ " b# " ++ show b
-   CacheBuildTreeRef t b  -> "build-tree-ref: " ++ (typeCodeFromRefType t:" ")
-                             ++ show b
-   CachePreference dep    -> "pref-ver: " ++ display dep
+showIndexCacheEntry entry = unwords $ case entry of
+   CachePackageId pkgid b -> [ packageKey
+                             , display (packageName pkgid)
+                             , display (packageVersion pkgid)
+                             , blocknoKey
+                             , show b
+                             ]
+   CacheBuildTreeRef t b  -> [ buildTreeRefKey
+                             , [typeCodeFromRefType t]
+                             , show b
+                             ]
+   CachePreference dep    -> [ preferredVersionKey
+                             , display dep
+                             ]
 
 readIndexCache :: BSS.ByteString -> [IndexCacheEntry]
 readIndexCache = mapMaybe readIndexCacheEntry . BSS.lines
