@@ -24,16 +24,19 @@ module Distribution.Client.Init (
 import System.IO
   ( hSetBuffering, stdout, BufferMode(..) )
 import System.Directory
-  ( getCurrentDirectory, doesDirectoryExist )
+  ( getCurrentDirectory, doesDirectoryExist, doesFileExist, copyFile
+  , getDirectoryContents, createDirectoryIfMissing )
 import System.FilePath
-  ( (</>) )
+  ( (</>), (<.>), takeBaseName )
 import Data.Time
   ( getCurrentTime, utcToLocalTime, toGregorian, localDay, getCurrentTimeZone )
 
+import Data.Char
+  ( toUpper )
 import Data.List
-  ( intersperse, intercalate, nub, groupBy, (\\) )
+  ( intercalate, nub, groupBy, (\\) )
 import Data.Maybe
-  ( fromMaybe, isJust, catMaybes )
+  ( fromMaybe, isJust, catMaybes, listToMaybe )
 import Data.Function
   ( on )
 import qualified Data.Map as M
@@ -42,39 +45,36 @@ import Data.Traversable
 import Control.Applicative
   ( (<$>) )
 import Control.Monad
-  ( when )
-#if MIN_VERSION_base(3,0,0)
-import Control.Monad
-  ( (>=>), join )
-#endif
+  ( when, unless, (>=>), join, forM_ )
 import Control.Arrow
-  ( (&&&) )
+  ( (&&&), (***) )
 
 import Text.PrettyPrint hiding (mode, cat)
 
 import Data.Version
   ( Version(..) )
 import Distribution.Version
-  ( orLaterVersion, withinVersion, VersionRange )
+  ( orLaterVersion, earlierVersion, intersectVersionRanges, VersionRange )
 import Distribution.Verbosity
   ( Verbosity )
 import Distribution.ModuleName
-  ( ModuleName, fromString )
+  ( ModuleName, fromString )  -- And for the Text instance
 import Distribution.InstalledPackageInfo
   ( InstalledPackageInfo, sourcePackageId, exposed )
 import qualified Distribution.Package as P
+import Language.Haskell.Extension ( Language(..) )
 
 import Distribution.Client.Init.Types
   ( InitFlags(..), PackageType(..), Category(..) )
 import Distribution.Client.Init.Licenses
-  ( bsd3, gplv2, gplv3, lgpl2, lgpl3, apache20 )
+  ( bsd2, bsd3, gplv2, gplv3, lgpl21, lgpl3, agplv3, apache20, mit, mpl20, isc )
 import Distribution.Client.Init.Heuristics
-  ( guessPackageName, guessAuthorNameMail, SourceFileEntry(..), scanForModules, neededBuildPrograms )
+  ( guessPackageName, guessAuthorNameMail, guessMainFileCandidates,
+    SourceFileEntry(..),
+    scanForModules, neededBuildPrograms )
 
 import Distribution.License
   ( License(..), knownLicenses )
-import Distribution.ModuleName
-  ( ) -- for the Text instance
 
 import Distribution.ReadE
   ( runReadE, readP_to_E )
@@ -87,7 +87,7 @@ import Distribution.Simple.Compiler
 import Distribution.Simple.Program
   ( ProgramConfiguration )
 import Distribution.Simple.PackageIndex
-  ( PackageIndex, moduleNameIndex )
+  ( InstalledPackageIndex, moduleNameIndex )
 import Distribution.Text
   ( display, Text(..) )
 
@@ -107,6 +107,7 @@ initCabal verbosity packageDBs comp conf initFlags = do
 
   writeLicense initFlags'
   writeSetupFile initFlags'
+  createSourceDirectories initFlags'
   success <- writeCabalFile initFlags'
 
   when success $ generateWarnings initFlags'
@@ -117,7 +118,7 @@ initCabal verbosity packageDBs comp conf initFlags = do
 
 -- | Fill in more details by guessing, discovering, or prompting the
 --   user.
-extendFlags :: PackageIndex -> InitFlags -> IO InitFlags
+extendFlags :: InstalledPackageIndex -> InitFlags -> IO InitFlags
 extendFlags pkgIx =
       getPackageName
   >=> getVersion
@@ -126,7 +127,9 @@ extendFlags pkgIx =
   >=> getHomepage
   >=> getSynopsis
   >=> getCategory
+  >=> getExtraSourceFiles
   >=> getLibOrExec
+  >=> getLanguage
   >=> getGenComments
   >=> getSrcDir
   >=> getModulesBuildToolsAndDeps pkgIx
@@ -153,7 +156,7 @@ getPackageName flags = do
               ?>> Just `fmap` (getCurrentDirectory >>= guessPackageName)
 
   pkgName' <-     return (flagToMaybe $ packageName flags)
-              ?>> maybePrompt flags (promptStr "Package name" guess)
+              ?>> maybePrompt flags (prompt "Package name" guess)
               ?>> return guess
 
   return $ flags { packageName = maybeToFlag pkgName' }
@@ -162,7 +165,7 @@ getPackageName flags = do
 --  if possible.
 getVersion :: InitFlags -> IO InitFlags
 getVersion flags = do
-  let v = Just $ Version { versionBranch = [0,1,0,0], versionTags = [] }
+  let v = Just $ Version [0,1,0,0] []
   v' <-     return (flagToMaybe $ version flags)
         ?>> maybePrompt flags (prompt "Package version" v)
         ?>> return v
@@ -178,13 +181,15 @@ getLicense flags = do
   return $ flags { license = maybeToFlag lic }
   where
     listedLicenses =
-      knownLicenses \\ [GPL Nothing, LGPL Nothing, Apache Nothing, OtherLicense]
+      knownLicenses \\ [GPL Nothing, LGPL Nothing, AGPL Nothing
+                       , Apache Nothing, OtherLicense]
 
 -- | The author's name and email. Prompt, or try to guess from an existing
 --   darcs repo.
 getAuthorInfo :: InitFlags -> IO InitFlags
 getAuthorInfo flags = do
-  (authorName, authorEmail)  <- (\(a,e) -> (flagToMaybe a, flagToMaybe e)) `fmap` guessAuthorNameMail
+  (authorName, authorEmail)  <-
+    (flagToMaybe *** flagToMaybe) `fmap` guessAuthorNameMail
   authorName'  <-     return (flagToMaybe $ author flags)
                   ?>> maybePrompt flags (promptStr "Author name" authorName)
                   ?>> return authorName
@@ -230,52 +235,111 @@ getCategory flags = do
                          (promptListOptional "Project category" [Codec ..]))
   return $ flags { category = maybeToFlag cat }
 
+-- | Try to guess extra source files (don't prompt the user).
+getExtraSourceFiles :: InitFlags -> IO InitFlags
+getExtraSourceFiles flags = do
+  extraSrcFiles <-     return (extraSrc flags)
+                   ?>> Just `fmap` guessExtraSourceFiles flags
+
+  return $ flags { extraSrc = extraSrcFiles }
+
+-- | Try to guess things to include in the extra-source-files field.
+--   For now, we just look for things in the root directory named
+--   'readme', 'changes', or 'changelog', with any sort of
+--   capitalization and any extension.
+guessExtraSourceFiles :: InitFlags -> IO [FilePath]
+guessExtraSourceFiles flags = do
+  dir <-
+    maybe getCurrentDirectory return . flagToMaybe $ packageDir flags
+  files <- getDirectoryContents dir
+  return $ filter isExtra files
+
+  where
+    isExtra = (`elem` ["README", "CHANGES", "CHANGELOG"])
+            . map toUpper
+            . takeBaseName
+
 -- | Ask whether the project builds a library or executable.
 getLibOrExec :: InitFlags -> IO InitFlags
 getLibOrExec flags = do
   isLib <-     return (flagToMaybe $ packageType flags)
            ?>> maybePrompt flags (either (const Library) id `fmap`
-                                   (promptList "What does the package build"
-                                               [Library, Executable]
-                                               Nothing display False))
+                                   promptList "What does the package build"
+                                   [Library, Executable]
+                                   Nothing display False)
            ?>> return (Just Library)
+  mainFile <- if isLib /= Just Executable then return Nothing else
+                    getMainFile flags
 
-  return $ flags { packageType = maybeToFlag isLib }
+  return $ flags { packageType = maybeToFlag isLib
+                 , mainIs = maybeToFlag mainFile
+                 }
 
--- | Ask whether to generate explanitory comments.
+-- | Try to guess the main file of the executable, and prompt the user to choose
+-- one of them. Top-level modules including the word 'Main' in the file name
+-- will be candidates, and shorter filenames will be preferred.
+getMainFile :: InitFlags -> IO (Maybe FilePath)
+getMainFile flags =
+  return (flagToMaybe $ mainIs flags)
+  ?>> do
+    candidates <- guessMainFileCandidates flags
+    let showCandidate = either (++" (does not yet exist)") id
+        defaultFile = listToMaybe candidates
+    maybePrompt flags (either id (either id id) `fmap`
+                       promptList "What is the main module of the executable"
+                       candidates
+                       defaultFile showCandidate True)
+      ?>> return (fmap (either id id) defaultFile)
+
+-- | Ask for the base language of the package.
+getLanguage :: InitFlags -> IO InitFlags
+getLanguage flags = do
+  lang <-     return (flagToMaybe $ language flags)
+          ?>> maybePrompt flags
+                (either UnknownLanguage id `fmap`
+                  promptList "What base language is the package written in"
+                  [Haskell2010, Haskell98]
+                  (Just Haskell2010) display True)
+          ?>> return (Just Haskell2010)
+
+  return $ flags { language = maybeToFlag lang }
+
+-- | Ask whether to generate explanatory comments.
 getGenComments :: InitFlags -> IO InitFlags
 getGenComments flags = do
-  genComments <-     return (flagToMaybe $ noComments flags)
+  genComments <-     return (not <$> flagToMaybe (noComments flags))
                  ?>> maybePrompt flags (promptYesNo promptMsg (Just False))
                  ?>> return (Just False)
   return $ flags { noComments = maybeToFlag (fmap not genComments) }
   where
     promptMsg = "Include documentation on what each field means (y/n)"
 
--- | Try to guess the source root directory (don't prompt the user).
+-- | Ask for the source root directory.
 getSrcDir :: InitFlags -> IO InitFlags
 getSrcDir flags = do
-  srcDirs <-     return (sourceDirs flags)
-             ?>> Just `fmap` (guessSourceDirs flags)
+  srcDirs <- return (sourceDirs flags)
+             ?>> fmap (:[]) `fmap` guessSourceDir flags
+             ?>> fmap (fmap ((:[]) . either id id) . join) (maybePrompt
+                      flags
+                      (promptListOptional' "Source directory" ["src"] id))
 
   return $ flags { sourceDirs = srcDirs }
 
--- | Try to guess source directories.  Could try harder; for the
+-- | Try to guess source directory. Could try harder; for the
 --   moment just looks to see whether there is a directory called 'src'.
-guessSourceDirs :: InitFlags -> IO [String]
-guessSourceDirs flags = do
-  dir      <- fromMaybe getCurrentDirectory
-                (fmap return . flagToMaybe $ packageDir flags)
+guessSourceDir :: InitFlags -> IO (Maybe String)
+guessSourceDir flags = do
+  dir      <-
+    maybe getCurrentDirectory return . flagToMaybe $ packageDir flags
   srcIsDir <- doesDirectoryExist (dir </> "src")
-  if srcIsDir
-    then return ["src"]
-    else return []
+  return $ if srcIsDir
+             then Just "src"
+             else Nothing
 
 -- | Get the list of exposed modules and extra tools needed to build them.
-getModulesBuildToolsAndDeps :: PackageIndex -> InitFlags -> IO InitFlags
+getModulesBuildToolsAndDeps :: InstalledPackageIndex -> InitFlags -> IO InitFlags
 getModulesBuildToolsAndDeps pkgIx flags = do
-  dir <- fromMaybe getCurrentDirectory
-                   (fmap return . flagToMaybe $ packageDir flags)
+  dir <- maybe getCurrentDirectory return . flagToMaybe $ packageDir flags
 
   -- XXX really should use guessed source roots.
   sourceFiles <- scanForModules dir
@@ -288,15 +352,26 @@ getModulesBuildToolsAndDeps pkgIx flags = do
 
   deps <-      return (dependencies flags)
            ?>> Just <$> importsToDeps flags
-                        (fromString "Prelude" : concatMap imports sourceFiles)
+                        (fromString "Prelude" :  -- to ensure we get base as a dep
+                           (   nub   -- only need to consider each imported package once
+                             . filter (`notElem` mods)  -- don't consider modules from
+                                                        -- this package itself
+                             . concatMap imports
+                             $ sourceFiles
+                           )
+                        )
                         pkgIx
 
+  exts <-     return (otherExts flags)
+          ?>> (return . Just . nub . concatMap extensions $ sourceFiles)
+
   return $ flags { exposedModules = Just mods
                  , buildTools     = tools
                  , dependencies   = deps
+                 , otherExts      = exts
                  }
 
-importsToDeps :: InitFlags -> [ModuleName] -> PackageIndex -> IO [P.Dependency]
+importsToDeps :: InitFlags -> [ModuleName] -> InstalledPackageIndex -> IO [P.Dependency]
 importsToDeps flags mods pkgIx = do
 
   let modMap :: M.Map ModuleName [InstalledPackageInfo]
@@ -333,7 +408,7 @@ chooseDep flags (m, Just ps)
       grps  -> do message flags ("\nWarning: multiple packages found providing "
                                  ++ display m
                                  ++ ": " ++ intercalate ", " (map (display . P.pkgName . head) grps))
-                  message flags ("You will need to pick one and manually add it to the Build-depends: field.")
+                  message flags "You will need to pick one and manually add it to the Build-depends: field."
                   return Nothing
   where
     pkgGroups = groupBy ((==) `on` P.pkgName) (map sourcePackageId ps)
@@ -351,7 +426,18 @@ chooseDep flags (m, Just ps)
                             (pvpize . maximum . map P.pkgVersion $ pids)
 
     pvpize :: Version -> VersionRange
-    pvpize v = withinVersion $ v { versionBranch = take 2 (versionBranch v) }
+    pvpize v = orLaterVersion v'
+               `intersectVersionRanges`
+               earlierVersion (incVersion 1 v')
+      where v' = (v { versionBranch = take 2 (versionBranch v) })
+
+incVersion :: Int -> Version -> Version
+incVersion n (Version vlist tags) = Version (incVersion' n vlist) tags
+  where
+    incVersion' 0 []     = [1]
+    incVersion' 0 (v:_)  = [v+1]
+    incVersion' m []     = replicate m 0 ++ [1]
+    incVersion' m (v:vs) = v : incVersion' (m-1) vs
 
 ---------------------------------------------------------------------------
 --  Prompting/user interaction  -------------------------------------------
@@ -416,12 +502,19 @@ promptListOptional :: (Text t, Eq t)
                    => String            -- ^ prompt
                    -> [t]               -- ^ choices
                    -> IO (Maybe (Either String t))
-promptListOptional pr choices =
+promptListOptional pr choices = promptListOptional' pr choices display
+
+promptListOptional' :: Eq t
+                   => String            -- ^ prompt
+                   -> [t]               -- ^ choices
+                   -> (t -> String)     -- ^ show an item
+                   -> IO (Maybe (Either String t))
+promptListOptional' pr choices displayItem =
     fmap rearrange
   $ promptList pr (Nothing : map Just choices) (Just Nothing)
-               (maybe "(none)" display) True
+               (maybe "(none)" displayItem) True
   where
-    rearrange = either (Just . Left) (maybe Nothing (Just . Right))
+    rearrange = either (Just . Left) (fmap Right)
 
 -- | Create a prompt from a list of items.
 promptList :: Eq t
@@ -435,8 +528,7 @@ promptList pr choices def displayItem other = do
   putStrLn $ pr ++ ":"
   let options1 = map (\c -> (Just c == def, displayItem c)) choices
       options2 = zip ([1..]::[Int])
-                     (options1 ++ if other then [(False, "Other (specify)")]
-                                           else [])
+                     (options1 ++ [(False, "Other (specify)") | other])
   mapM_ (putStrLn . \(n,(i,s)) -> showOption n i ++ s) options2
   promptList' displayItem (length options2) choices def other
  where showOption n i | n < 10 = " " ++ star i ++ " " ++ rest
@@ -474,14 +566,15 @@ readMaybe s = case reads s of
 writeLicense :: InitFlags -> IO ()
 writeLicense flags = do
   message flags "\nGenerating LICENSE..."
-  year <- getYear
+  year <- show <$> getYear
+  let authors = fromMaybe "???" . flagToMaybe . author $ flags
   let licenseFile =
         case license flags of
-          Flag BSD3 -> Just $ bsd3 (fromMaybe "???"
-                                  . flagToMaybe
-                                  . author
-                                  $ flags)
-                              (show year)
+          Flag BSD2
+            -> Just $ bsd2 authors year
+
+          Flag BSD3
+            -> Just $ bsd3 authors year
 
           Flag (GPL (Just (Version {versionBranch = [2]})))
             -> Just gplv2
@@ -489,19 +582,31 @@ writeLicense flags = do
           Flag (GPL (Just (Version {versionBranch = [3]})))
             -> Just gplv3
 
-          Flag (LGPL (Just (Version {versionBranch = [2]})))
-            -> Just lgpl2
+          Flag (LGPL (Just (Version {versionBranch = [2, 1]})))
+            -> Just lgpl21
 
           Flag (LGPL (Just (Version {versionBranch = [3]})))
             -> Just lgpl3
 
+          Flag (AGPL (Just (Version {versionBranch = [3]})))
+            -> Just agplv3
+
           Flag (Apache (Just (Version {versionBranch = [2, 0]})))
             -> Just apache20
 
+          Flag MIT
+            -> Just $ mit authors year
+
+          Flag (MPL (Version {versionBranch = [2, 0]}))
+            -> Just mpl20
+
+          Flag ISC
+            -> Just $ isc authors year
+
           _ -> Nothing
 
   case licenseFile of
-    Just licenseText -> writeFile "LICENSE" licenseText
+    Just licenseText -> writeFileSafe flags "LICENSE" licenseText
     Nothing -> message flags "Warning: unknown license type, you must put a copy in LICENSE yourself."
 
 getYear :: IO Integer
@@ -515,25 +620,56 @@ getYear = do
 writeSetupFile :: InitFlags -> IO ()
 writeSetupFile flags = do
   message flags "Generating Setup.hs..."
-  writeFile "Setup.hs" setupFile
+  writeFileSafe flags "Setup.hs" setupFile
  where
   setupFile = unlines
     [ "import Distribution.Simple"
     , "main = defaultMain"
     ]
 
--- XXX ought to do something sensible if a .cabal file already exists,
--- instead of overwriting.
 writeCabalFile :: InitFlags -> IO Bool
 writeCabalFile flags@(InitFlags{packageName = NoFlag}) = do
   message flags "Error: no package name provided."
   return False
 writeCabalFile flags@(InitFlags{packageName = Flag p}) = do
-  let cabalFileName = p ++ ".cabal"
+  let cabalFileName = display p ++ ".cabal"
   message flags $ "Generating " ++ cabalFileName ++ "..."
-  writeFile cabalFileName (generateCabalFile cabalFileName flags)
+  writeFileSafe flags cabalFileName (generateCabalFile cabalFileName flags)
   return True
 
+-- | Write a file \"safely\", backing up any existing version (unless
+--   the overwrite flag is set).
+writeFileSafe :: InitFlags -> FilePath -> String -> IO ()
+writeFileSafe flags fileName content = do
+  moveExistingFile flags fileName
+  writeFile fileName content
+
+-- | Create source directories, if they were given.
+createSourceDirectories :: InitFlags -> IO ()
+createSourceDirectories flags = case sourceDirs flags of
+                                  Just dirs -> forM_ dirs (createDirectoryIfMissing True)
+                                  Nothing   -> return ()
+
+-- | Move an existing file, if there is one, and the overwrite flag is
+--   not set.
+moveExistingFile :: InitFlags -> FilePath -> IO ()
+moveExistingFile flags fileName =
+  unless (overwrite flags == Flag True) $ do
+    e <- doesFileExist fileName
+    when e $ do
+      newName <- findNewName fileName
+      message flags $ "Warning: " ++ fileName ++ " already exists, backing up old version in " ++ newName
+      copyFile fileName newName
+
+findNewName :: FilePath -> IO FilePath
+findNewName oldName = findNewName' 0
+  where
+    findNewName' :: Integer -> IO FilePath
+    findNewName' n = do
+      let newName = oldName <.> ("save" ++ show n)
+      e <- doesFileExist newName
+      if e then findNewName' (n+1) else return newName
+
 -- | Generate a .cabal file from an InitFlags structure.  NOTE: this
 --   is rather ad-hoc!  What we would REALLY like is to have a
 --   standard low-level AST type representing .cabal files, which
@@ -545,14 +681,14 @@ writeCabalFile flags@(InitFlags{packageName = Flag p}) = do
 generateCabalFile :: String -> InitFlags -> String
 generateCabalFile fileName c =
   renderStyle style { lineLength = 79, ribbonsPerLine = 1.1 } $
-  (if (minimal c /= Flag True)
+  (if minimal c /= Flag True
     then showComment (Just $ "Initial " ++ fileName ++ " generated by cabal "
                           ++ "init.  For further documentation, see "
                           ++ "http://haskell.org/cabal/users-guide/")
          $$ text ""
     else empty)
   $$
-  vcat [ fieldS "name"          (packageName   c)
+  vcat [ field  "name"          (packageName   c)
                 (Just "The name of the package.")
                 True
 
@@ -607,22 +743,24 @@ generateCabalFile fileName c =
                 Nothing
                 True
 
-       , fieldS "extra-source-files" NoFlag
+       , fieldS "extra-source-files" (listFieldS (extraSrc c))
                 (Just "Extra files to be distributed with the package, such as examples or a README.")
-                False
+                True
 
-       , field  "cabal-version" (Flag $ orLaterVersion (Version [1,8] []))
+       , field  "cabal-version" (Flag $ orLaterVersion (Version [1,10] []))
                 (Just "Constraint on the version of Cabal needed to build this package.")
                 False
 
        , case packageType c of
            Flag Executable ->
-             text "\nexecutable" <+> text (fromMaybe "" . flagToMaybe $ packageName c) $$ (nest 2 $ vcat
-             [ fieldS "main-is" NoFlag (Just ".hs or .lhs file containing the Main module.") True
+             text "\nexecutable" <+>
+             text (maybe "" display . flagToMaybe $ packageName c) $$
+             nest 2 (vcat
+             [ fieldS "main-is" (mainIs c) (Just ".hs or .lhs file containing the Main module.") True
 
              , generateBuildInfo Executable c
              ])
-           Flag Library    -> text "\nlibrary" $$ (nest 2 $ vcat
+           Flag Library    -> text "\nlibrary" $$ nest 2 (vcat
              [ fieldS "exposed-modules" (listField (exposedModules c))
                       (Just "Modules exported by the library.")
                       True
@@ -640,24 +778,32 @@ generateCabalFile fileName c =
                  Executable -> "Modules included in this executable, other than Main.")
               True
 
+     , fieldS "other-extensions" (listField (otherExts c'))
+              (Just "LANGUAGE extensions used by modules in this package.")
+              True
+
      , fieldS "build-depends" (listField (dependencies c'))
               (Just "Other library packages from which modules are imported.")
               True
 
      , fieldS "hs-source-dirs" (listFieldS (sourceDirs c'))
               (Just "Directories containing source files.")
-              False
+              True
 
      , fieldS "build-tools" (listFieldS (buildTools c'))
               (Just "Extra tools (e.g. alex, hsc2hs, ...) needed to build the source.")
               False
+
+     , field  "default-language" (language c')
+              (Just "Base language which the package is written in.")
+              True
      ]
 
    listField :: Text s => Maybe [s] -> Flag String
    listField = listFieldS . fmap (map display)
 
    listFieldS :: Maybe [String] -> Flag String
-   listFieldS = Flag . maybe "" (concat . intersperse ", ")
+   listFieldS = Flag . maybe "" (intercalate ", ")
 
    field :: Text t => String -> Flag t -> Maybe String -> Bool -> Doc
    field s f = fieldS s (fmap display f)
@@ -676,15 +822,15 @@ generateCabalFile fileName c =
                         (False, _, _)     -> ($$ text "")
                       $
                       comment f <> text s <> colon
-                                <> text (take (20 - length s) (repeat ' '))
+                                <> text (replicate (20 - length s) ' ')
                                 <> text (fromMaybe "" . flagToMaybe $ f)
    comment NoFlag    = text "-- "
    comment (Flag "") = text "-- "
    comment _         = text ""
 
    showComment :: Maybe String -> Doc
-   showComment (Just t) = vcat . map text
-                        . map ("-- "++) . lines
+   showComment (Just t) = vcat
+                        . map (text . ("-- "++)) . lines
                         . renderStyle style {
                             lineLength = 76,
                             ribbonsPerLine = 1.05
@@ -714,9 +860,3 @@ generateWarnings flags = do
 message :: InitFlags -> String -> IO ()
 message (InitFlags{quiet = Flag True}) _ = return ()
 message _ s = putStrLn s
-
-#if MIN_VERSION_base(3,0,0)
-#else
-(>=>)       :: Monad m => (a -> m b) -> (b -> m c) -> (a -> m c)
-f >=> g     = \x -> f x >>= g
-#endif
