@@ -24,6 +24,8 @@ module Distribution.Client.InstallPlan (
   completed,
   failed,
   remove,
+  showPlanIndex,
+  showInstallPlan,
 
   -- ** Query functions
   planPlatform,
@@ -49,10 +51,11 @@ import Distribution.Client.Types
          ( SourcePackage(packageDescription), ConfiguredPackage(..)
          , ReadyPackage(..), readyPackageToConfiguredPackage
          , InstalledPackage, BuildFailure, BuildSuccess(..), enableStanzas
-         , InstalledPackage (..) )
+         , InstalledPackage(..), fakeInstalledPackageId )
 import Distribution.Package
          ( PackageIdentifier(..), PackageName(..), Package(..), packageName
-         , PackageFixedDeps(..), Dependency(..) )
+         , PackageFixedDeps(..), Dependency(..), InstalledPackageId
+         , PackageInstalled(..) )
 import Distribution.Version
          ( Version, withinRange )
 import Distribution.PackageDescription
@@ -62,15 +65,15 @@ import Distribution.Client.PackageUtils
          ( externalBuildDepends )
 import Distribution.PackageDescription.Configuration
          ( finalizePackageDescription )
-import Distribution.Client.PackageIndex
-         ( PackageIndex )
-import qualified Distribution.Client.PackageIndex as PackageIndex
+import Distribution.Simple.PackageIndex
+         ( PackageIndex, FakeMap )
+import qualified Distribution.Simple.PackageIndex as PackageIndex
 import Distribution.Text
          ( display )
 import Distribution.System
          ( Platform )
 import Distribution.Compiler
-         ( CompilerId(..) )
+         ( CompilerInfo(..) )
 import Distribution.Client.Utils
          ( duplicates, duplicatesBy, mergeBy, MergeResult(..) )
 import Distribution.Simple.Utils
@@ -85,6 +88,10 @@ import qualified Data.Graph as Graph
 import Data.Graph (Graph)
 import Control.Exception
          ( assert )
+import Data.Maybe (catMaybes)
+import qualified Data.Map as Map
+
+type PlanIndex = PackageIndex PlanPackage
 
 -- When cabal tries to install a number of packages, including all their
 -- dependencies it has a non-trivial problem to solve.
@@ -150,40 +157,89 @@ instance PackageFixedDeps PlanPackage where
   depends (Installed pkg _) = depends pkg
   depends (Failed    pkg _) = depends pkg
 
+instance PackageInstalled PlanPackage where
+  installedPackageId (PreExisting pkg)   = installedPackageId pkg
+  installedPackageId (Configured  pkg)   = installedPackageId pkg
+  installedPackageId (Processing  pkg)   = installedPackageId pkg
+  -- NB: defer to the actual installed package info in this case
+  installedPackageId (Installed _ (BuildOk _ _ (Just ipkg))) = installedPackageId ipkg
+  installedPackageId (Installed   pkg _) = installedPackageId pkg
+  installedPackageId (Failed      pkg _) = installedPackageId pkg
+
+  installedDepends (PreExisting pkg) = installedDepends pkg
+  installedDepends (Configured  pkg) = installedDepends pkg
+  installedDepends (Processing pkg)  = installedDepends pkg
+  installedDepends (Installed _ (BuildOk _ _ (Just ipkg))) = installedDepends ipkg
+  installedDepends (Installed pkg _) = installedDepends pkg
+  installedDepends (Failed    pkg _) = installedDepends pkg
+
 data InstallPlan = InstallPlan {
-    planIndex    :: PackageIndex PlanPackage,
+    planIndex    :: PlanIndex,
+    planFakeMap  :: FakeMap,
     planGraph    :: Graph,
     planGraphRev :: Graph,
     planPkgOf    :: Graph.Vertex -> PlanPackage,
-    planVertexOf :: PackageIdentifier -> Graph.Vertex,
+    planVertexOf :: InstalledPackageId -> Graph.Vertex,
     planPlatform :: Platform,
-    planCompiler :: CompilerId
+    planCompiler :: CompilerInfo
   }
 
 invariant :: InstallPlan -> Bool
 invariant plan =
-  valid (planPlatform plan) (planCompiler plan) (planIndex plan)
+  valid (planPlatform plan) (planCompiler plan) (planFakeMap plan) (planIndex plan)
 
 internalError :: String -> a
 internalError msg = error $ "InstallPlan: internal error: " ++ msg
 
+showPlanIndex :: PlanIndex -> String
+showPlanIndex index =
+    intercalate "\n" (map showPlanPackage (PackageIndex.allPackages index))
+  where showPlanPackage p =
+            showPlanPackageTag p ++ " "
+                ++ display (packageId p) ++ " ("
+                ++ display (installedPackageId p) ++ ")"
+
+showInstallPlan :: InstallPlan -> String
+showInstallPlan plan =
+    showPlanIndex (planIndex plan) ++ "\n" ++
+    "fake map:\n  " ++ intercalate "\n  " (map showKV (Map.toList (planFakeMap plan)))
+  where showKV (k,v) = display k ++ " -> " ++ display v
+
+showPlanPackageTag :: PlanPackage -> String
+showPlanPackageTag (PreExisting _) = "PreExisting"
+showPlanPackageTag (Configured _)  = "Configured"
+showPlanPackageTag (Processing _)  = "Processing"
+showPlanPackageTag (Installed _ _) = "Installed"
+showPlanPackageTag (Failed _ _)    = "Failed"
+
 -- | Build an installation plan from a valid set of resolved packages.
 --
-new :: Platform -> CompilerId -> PackageIndex PlanPackage
+new :: Platform -> CompilerInfo -> PlanIndex
     -> Either [PlanProblem] InstallPlan
-new platform compiler index =
-  case problems platform compiler index of
+new platform cinfo index =
+  -- NB: Need to pre-initialize the fake-map with pre-existing
+  -- packages
+  let isPreExisting (PreExisting _) = True
+      isPreExisting _ = False
+      fakeMap = Map.fromList
+              . map (\p -> (fakeInstalledPackageId (packageId p), installedPackageId p))
+              . filter isPreExisting
+              $ PackageIndex.allPackages index in
+  case problems platform cinfo fakeMap index of
     [] -> Right InstallPlan {
             planIndex    = index,
+            planFakeMap  = fakeMap,
             planGraph    = graph,
             planGraphRev = Graph.transposeG graph,
             planPkgOf    = vertexToPkgId,
             planVertexOf = fromMaybe noSuchPkgId . pkgIdToVertex,
             planPlatform = platform,
-            planCompiler = compiler
+            planCompiler = cinfo
           }
       where (graph, vertexToPkgId, pkgIdToVertex) =
               PackageIndex.dependencyGraph index
+              -- NB: doesn't need to know planFakeMap because the
+              -- fakemap is empty at this point.
             noSuchPkgId = internalError "package is not in the graph"
     probs -> Left probs
 
@@ -227,11 +283,13 @@ ready plan = assert check readyPackages
       ]
 
     hasAllInstalledDeps :: ConfiguredPackage -> Maybe [Installed.InstalledPackageInfo]
-    hasAllInstalledDeps = mapM isInstalledDep . depends
+    hasAllInstalledDeps = mapM isInstalledDep . installedDepends
 
-    isInstalledDep :: PackageIdentifier -> Maybe Installed.InstalledPackageInfo
+    isInstalledDep :: InstalledPackageId -> Maybe Installed.InstalledPackageInfo
     isInstalledDep pkgid =
-      case PackageIndex.lookupPackageId (planIndex plan) pkgid of
+      -- NB: Need to check if the ID has been updated in planFakeMap, in which case we
+      -- might be dealing with an old pointer
+      case PackageIndex.fakeLookupInstalledPackageId (planFakeMap plan) (planIndex plan) pkgid of
         Just (Configured  _)                            -> Nothing
         Just (Processing  _)                            -> Nothing
         Just (Failed    _ _)                            -> internalError depOnFailed
@@ -261,15 +319,25 @@ processing pkgs plan = assert (invariant plan') plan'
 -- * The package must exist in the graph and be in the processing state.
 -- * The package must have had no uninstalled dependent packages.
 --
-completed :: PackageIdentifier
+completed :: InstalledPackageId
           -> BuildSuccess
           -> InstallPlan -> InstallPlan
 completed pkgid buildResult plan = assert (invariant plan') plan'
   where
     plan'     = plan {
-                  planIndex = PackageIndex.insert installed (planIndex plan)
+                  -- NB: installation can change the IPID, so better
+                  -- record it in the fake mapping...
+                  planFakeMap = insert_fake_mapping buildResult
+                              $ planFakeMap plan,
+                  planIndex = PackageIndex.insert installed
+                            . PackageIndex.deleteInstalledPackageId pkgid
+                            $ planIndex plan
                 }
+    -- ...but be sure to use the *old* IPID for the lookup for the
+    -- preexisting record
     installed = Installed (lookupProcessingPackage plan pkgid) buildResult
+    insert_fake_mapping (BuildOk _ _ (Just ipi)) = Map.insert pkgid (installedPackageId ipi)
+    insert_fake_mapping _ = id
 
 -- | Marks a package in the graph as having failed. It also marks all the
 -- packages that depended on it as having failed.
@@ -277,13 +345,14 @@ completed pkgid buildResult plan = assert (invariant plan') plan'
 -- * The package must exist in the graph and be in the processing
 -- state.
 --
-failed :: PackageIdentifier -- ^ The id of the package that failed to install
+failed :: InstalledPackageId -- ^ The id of the package that failed to install
        -> BuildFailure      -- ^ The build result to use for the failed package
        -> BuildFailure      -- ^ The build result to use for its dependencies
        -> InstallPlan
        -> InstallPlan
 failed pkgid buildResult buildResult' plan = assert (invariant plan') plan'
   where
+    -- NB: failures don't update IPIDs
     plan'    = plan {
                  planIndex = PackageIndex.merge (planIndex plan) failures
                }
@@ -297,18 +366,21 @@ failed pkgid buildResult buildResult' plan = assert (invariant plan') plan'
 -- | Lookup the reachable packages in the reverse dependency graph.
 --
 packagesThatDependOn :: InstallPlan
-                     -> PackageIdentifier -> [PlanPackage]
-packagesThatDependOn plan = map (planPkgOf plan)
+                     -> InstalledPackageId -> [PlanPackage]
+packagesThatDependOn plan pkgid = map (planPkgOf plan)
                           . tail
                           . Graph.reachable (planGraphRev plan)
                           . planVertexOf plan
+                          $ Map.findWithDefault pkgid pkgid (planFakeMap plan)
 
 -- | Lookup a package that we expect to be in the processing state.
 --
 lookupProcessingPackage :: InstallPlan
-                        -> PackageIdentifier -> ReadyPackage
+                        -> InstalledPackageId -> ReadyPackage
 lookupProcessingPackage plan pkgid =
-  case PackageIndex.lookupPackageId (planIndex plan) pkgid of
+  -- NB: processing packages are guaranteed to not indirect through
+  -- planFakeMap
+  case PackageIndex.lookupInstalledPackageId (planIndex plan) pkgid of
     Just (Processing pkg) -> pkg
     _  -> internalError $ "not in processing state or no such pkg " ++ display pkgid
 
@@ -330,8 +402,8 @@ checkConfiguredPackage pkg                =
 --
 -- * if the result is @False@ use 'problems' to get a detailed list.
 --
-valid :: Platform -> CompilerId -> PackageIndex PlanPackage -> Bool
-valid platform comp index = null (problems platform comp index)
+valid :: Platform -> CompilerInfo -> FakeMap -> PlanIndex -> Bool
+valid platform cinfo fakeMap index = null (problems platform cinfo fakeMap index)
 
 data PlanProblem =
      PackageInvalid       ConfiguredPackage [PackageProblem]
@@ -381,26 +453,26 @@ showPlanProblem (PackageStateInvalid pkg pkg') =
 -- error messages. This is mainly intended for debugging purposes.
 -- Use 'showPlanProblem' for a human readable explanation.
 --
-problems :: Platform -> CompilerId
-         -> PackageIndex PlanPackage -> [PlanProblem]
-problems platform comp index =
+problems :: Platform -> CompilerInfo -> FakeMap
+         -> PlanIndex -> [PlanProblem]
+problems platform cinfo fakeMap index =
      [ PackageInvalid pkg packageProblems
      | Configured pkg <- PackageIndex.allPackages index
-     , let packageProblems = configuredPackageProblems platform comp pkg
+     , let packageProblems = configuredPackageProblems platform cinfo pkg
      , not (null packageProblems) ]
 
-  ++ [ PackageMissingDeps pkg missingDeps
-     | (pkg, missingDeps) <- PackageIndex.brokenPackages index ]
+  ++ [ PackageMissingDeps pkg (catMaybes (map (fmap packageId . PackageIndex.fakeLookupInstalledPackageId fakeMap index) missingDeps))
+     | (pkg, missingDeps) <- PackageIndex.brokenPackages' fakeMap index ]
 
   ++ [ PackageCycle cycleGroup
-     | cycleGroup <- PackageIndex.dependencyCycles index ]
+     | cycleGroup <- PackageIndex.dependencyCycles' fakeMap index ]
 
   ++ [ PackageInconsistency name inconsistencies
-     | (name, inconsistencies) <- PackageIndex.dependencyInconsistencies index ]
+     | (name, inconsistencies) <- PackageIndex.dependencyInconsistencies' fakeMap index ]
 
   ++ [ PackageStateInvalid pkg pkg'
      | pkg <- PackageIndex.allPackages index
-     , Just pkg' <- map (PackageIndex.lookupPackageId index) (depends pkg)
+     , Just pkg' <- map (PackageIndex.fakeLookupInstalledPackageId fakeMap index) (installedDepends pkg)
      , not (stateDependencyRelation pkg pkg') ]
 
 -- | The graph of packages (nodes) and dependencies (edges) must be acyclic.
@@ -408,7 +480,7 @@ problems platform comp index =
 -- * if the result is @False@ use 'PackageIndex.dependencyCycles' to find out
 --   which packages are involved in dependency cycles.
 --
-acyclic :: PackageIndex PlanPackage -> Bool
+acyclic :: PlanIndex -> Bool
 acyclic = null . PackageIndex.dependencyCycles
 
 -- | An installation plan is closed if for every package in the set, all of
@@ -418,7 +490,7 @@ acyclic = null . PackageIndex.dependencyCycles
 -- * if the result is @False@ use 'PackageIndex.brokenPackages' to find out
 --   which packages depend on packages not in the index.
 --
-closed :: PackageIndex PlanPackage -> Bool
+closed :: PlanIndex -> Bool
 closed = null . PackageIndex.brokenPackages
 
 -- | An installation plan is consistent if all dependencies that target a
@@ -437,7 +509,7 @@ closed = null . PackageIndex.brokenPackages
 -- * if the result is @False@ use 'PackageIndex.dependencyInconsistencies' to
 --   find out which packages are.
 --
-consistent :: PackageIndex PlanPackage -> Bool
+consistent :: PlanIndex -> Bool
 consistent = null . PackageIndex.dependencyInconsistencies
 
 -- | The states of packages have that depend on each other must respect
@@ -473,9 +545,9 @@ stateDependencyRelation _               _               = False
 -- in the configuration given by the flag assignment, all the package
 -- dependencies are satisfied by the specified packages.
 --
-configuredPackageValid :: Platform -> CompilerId -> ConfiguredPackage -> Bool
-configuredPackageValid platform comp pkg =
-  null (configuredPackageProblems platform comp pkg)
+configuredPackageValid :: Platform -> CompilerInfo -> ConfiguredPackage -> Bool
+configuredPackageValid platform cinfo pkg =
+  null (configuredPackageProblems platform cinfo pkg)
 
 data PackageProblem = DuplicateFlag FlagName
                     | MissingFlag   FlagName
@@ -513,9 +585,9 @@ showPackageProblem (InvalidDep dep pkgid) =
   ++ " but the configuration specifies " ++ display pkgid
   ++ " which does not satisfy the dependency."
 
-configuredPackageProblems :: Platform -> CompilerId
+configuredPackageProblems :: Platform -> CompilerInfo
                           -> ConfiguredPackage -> [PackageProblem]
-configuredPackageProblems platform comp
+configuredPackageProblems platform cinfo
   (ConfiguredPackage pkg specifiedFlags stanzas specifiedDeps) =
      [ DuplicateFlag flag | ((flag,_):_) <- duplicates specifiedFlags ]
   ++ [ MissingFlag flag | OnlyInLeft  flag <- mergedFlags ]
@@ -548,7 +620,7 @@ configuredPackageProblems platform comp
       --TODO: use something lower level than finalizePackageDescription
       case finalizePackageDescription specifiedFlags
          (const True)
-         platform comp
+         platform cinfo
          []
          (enableStanzas stanzas $ packageDescription pkg) of
         Right (resolvedPkg, _) -> externalBuildDepends resolvedPkg
