@@ -39,8 +39,9 @@ import Distribution.Client.ParseUtils  ( parseFields, ppFields, ppSection )
 import Distribution.Client.Setup       ( GlobalFlags(..), ConfigExFlags(..)
                                        , InstallFlags(..)
                                        , defaultSandboxLocation )
+import Distribution.Utils.NubList            ( toNubList )
 import Distribution.Simple.Compiler    ( Compiler, PackageDB(..)
-                                       , compilerFlavor, showCompilerId )
+                                       , compilerFlavor, showCompilerIdWithAbi )
 import Distribution.Simple.InstallDirs ( InstallDirs(..), PathTemplate
                                        , defaultInstallDirs, combineInstallDirs
                                        , fromPathTemplate, toPathTemplate )
@@ -200,12 +201,12 @@ initialPackageEnvironment sandboxDir compiler platform = do
        savedUserInstallDirs   = installDirs,
        savedGlobalInstallDirs = installDirs,
        savedGlobalFlags = (savedGlobalFlags initialConfig) {
-          globalLocalRepos = [sandboxDir </> "packages"]
+          globalLocalRepos = toNubList [sandboxDir </> "packages"]
           },
        savedConfigureFlags = setPackageDB sandboxDir compiler platform
                              (savedConfigureFlags initialConfig),
        savedInstallFlags = (savedInstallFlags initialConfig) {
-         installSummaryFile = [toPathTemplate (sandboxDir </>
+         installSummaryFile = toNubList [toPathTemplate (sandboxDir </>
                                                "logs" </> "build.log")]
          }
        }
@@ -216,9 +217,11 @@ sandboxPackageDBPath :: FilePath -> Compiler -> Platform -> String
 sandboxPackageDBPath sandboxDir compiler platform =
     sandboxDir
          </> (Text.display platform ++ "-"
-             ++ showCompilerId compiler
+             ++ showCompilerIdWithAbi compiler
              ++ "-packages.conf.d")
-
+-- The path in sandboxPackageDBPath should be kept in sync with the
+-- path in the bootstrap.sh which is used to bootstrap cabal-install
+-- into a sandbox.
 
 -- | Use the package DB location specific for this compiler.
 setPackageDB :: FilePath -> Compiler -> Platform -> ConfigFlags -> ConfigFlags
@@ -325,7 +328,8 @@ tryLoadSandboxPackageEnvironmentFile verbosity pkgEnvFile configFileFlag = do
   pkgEnv <- handleParseResult verbosity pkgEnvFile minp
 
   -- Get the saved sandbox directory.
-  -- TODO: Use substPathTemplate with compilerTemplateEnv ++ platformTemplateEnv.
+  -- TODO: Use substPathTemplate with
+  -- compilerTemplateEnv ++ platformTemplateEnv ++ abiTemplateEnv.
   let sandboxDir = fromFlagOrDefault defaultSandboxLocation
                    . fmap fromPathTemplate . prefix . savedUserInstallDirs
                    . pkgEnvSavedConfig $ pkgEnv
@@ -470,10 +474,7 @@ parsePackageEnvironment initial str = do
           configProgramPaths  = paths,
           configProgramArgs   = args
           },
-       savedHaddockFlags      = haddockFlags {
-         haddockProgramPaths  = paths,
-         haddockProgramArgs   = args
-         },
+       savedHaddockFlags      = haddockFlags,
        savedUserInstallDirs   = installDirs,
        savedGlobalInstallDirs = installDirs
        }
